/*
 * Hardware initialization.
 *
 * $LastChangedRevision: 483 $
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include "Target.h"
#include "Main.h"
#include "Hardware.h"

/* private prototypes */
static void sendSerial(tSerial* serial, uint8 data);
static uint8 receiveSerial(tSerial* serial);
static uint8 isReceiveDataAvailable(tSerial* serial);
static uint8 isSendReady(tSerial* serial);

/* private macros */
#define F_FRC           8000000         /* fast RC internal oscillator frequency after tuning */
#define F_IN            F_FRC           /* FRC as input for PLL */
// ChB #define F_OSC           F_IN*40/(2*2)   /* fOSC = fIN*M/(N1*N2): (PLL) oscillator frequency */
#define F_OSC           F_IN*70/(2*2)   /* fOSC = fIN*M/(N1*N2): (PLL) oscillator frequency */
#define F_CY            F_OSC/2         /* instruction cycle frequency */

#define BAUDRATE        115200          /* baudrate for serial interface */
#define BRGVAL          ((F_CY/BAUDRATE)/16)-1  // for BRGH = 0
//#define BRGVAL          ((F_CY/BAUDRATE)/4)-1  // for BRGH = 1

 
 // FICD
#pragma config ICS = PGD3               // ICD Communication Channel Select bits (Communicate on PGEC1 and PGED1)
#pragma config JTAGEN = OFF             // JTAG Enable bit (JTAG is disabled)

// FPOR
#pragma config ALTI2C1 = OFF            // Alternate I2C1 pins (I2C1 mapped to SDA1/SCL1 pins)
#pragma config ALTI2C2 = OFF            // Alternate I2C2 pins (I2C2 mapped to SDA2/SCL2 pins)
#pragma config WDTWIN = WIN25           // Watchdog Window Select bits (WDT Window is 25% of WDT period)

// FWDT
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler bits (1:32,768)
#pragma config WDTPRE = PR128           // Watchdog Timer Prescaler bit (1:128)
#pragma config PLLKEN = ON              // PLL Lock Enable bit (Clock switch to PLL source will wait until the PLL lock signal is valid.)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable bit (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF              // Watchdog Timer Enable bit (Watchdog timer always enabled)

// FOSC
#pragma config POSCMD = XT            // Primary Oscillator Mode Select bits (Primary Oscillator disabled)
// ChB #pragma config POSCMD = NONE            // Primary Oscillator Mode Select bits (Primary Oscillator disabled)

#pragma config OSCIOFNC = OFF           // OSC2 Pin Function bit (OSC2 is clock output)
#pragma config IOL1WAY = ON             // Peripheral pin select configuration (Allow only one reconfiguration)
#pragma config FCKSM = CSECMD           // Clock Switching Mode bits (Clock switching is enabled,Fail-safe Clock Monitor is disabled)

// FOSCSEL
#pragma config FNOSC = FRCPLL           // Oscillator Source Selection (Fast RC Oscillator with divide-by-N with PLL module (FRCPLL))
#pragma config PWMLOCK = ON             // PWM Lock Enable bit (Certain PWM registers may only be written after key sequence)
#pragma config IESO = ON                // Two-speed Oscillator Start-up Enable bit (Start up device with FRC, then switch to user-selected oscillator source)

// FGS
#pragma config GWRP = OFF               // General Segment Write-Protect bit (General Segment may be written)
#pragma config GCP = OFF                // General Segment Code-Protect bit (General Segment Code protect is Disabled)

/*
 * Hardware initialization.
 */
void initHardware(void)
{   
    //The settings below set up the oscillator and PLL for x MIPS as
    //follows:
    //                  Fin  * M
    // Fosc       =     ---------
    //                   N2 * N1   
    //
    // Fin   	  = 7.37 MHz (Internal Oscillator) or 8MHZ ( Crystal Oscillator)
    // Fosc       = x MHz
    // Fcy        = Fosc/2 = 
    /****************** Clock definitions *********************************/
    // 70 MIPS (70.00 Mhz) (i.e 8 * (70/4)) Clock Configuration using External Oscillator
    PLLFBD = 68; // M = 70
    CLKDIVbits.PLLPOST = 0; // N1=2
    CLKDIVbits.PLLPRE = 0; // N2=2


    // Initiate Clock Switch to Primary Oscillator with PLL (NOSC=0b011)

    __builtin_write_OSCCONH(0x03);
    __builtin_write_OSCCONL(0x01);

    while (OSCCONbits.COSC != 0x03);

    while (OSCCONbits.LOCK != 1);
 
    /*------------------------------------------------------------------------*/
    /* Configuration of INTOSC oscillator */
    /*------------------------------------------------------------------------*/
    /* Configure PLL prescaler, PLL postscaler, PLL divisor
     * FRC frequency: 7.37MHz -> tuning to 8MHz
     * NOTE: FRC frequency is not accurate -> frequency may vary!
     * fCY = 8MHz * 40/(2*2) / 2 = 40MHz
     */   
//    OSCTUNbits.TUN = 23;        /* 7.37MHz + 23*0.00375*7.37MHz = 8MHz */
// 
//    PLLFBD = 40;                /* M = 40 */
//    
//    CLKDIVbits.PLLPOST = 0;     /* N2 = 2 */
//    CLKDIVbits.PLLPRE = 0;      /* N1 = 2 */
//
//    /* Switch to FRC with PLL */
//    __builtin_write_OSCCONH(0x01);
//    __builtin_write_OSCCONL(0x01);
//    /* Wait for Clock switch to occur */
//    while (OSCCONbits.COSC != 0b001);
//    /* Wait for PLL to lock */
//    while(OSCCONbits.LOCK != 1) {};
 
    /* Enable watchdog */
    RCONbits.SWDTEN = 1;
    
    /*------------------------------------------------------------------------*/
    /* Configuration of serial port */
    /*------------------------------------------------------------------------*/
    // MOD dsPIC33FJ -> dsPIC33EP mapping corrected
    // RX has to be PIN21, dsPIC33FJ -> RP10, dsPIC33EP -> RP43
    RPINR18bits.U1RXR = 34;
    // TX has to be PIN22, dsPIC33FJ -> RP11, dsPIC33EP -> RP42
    RPOR0bits.RP35R = 1;       /* map UART #1 tx to RP42 */

    
    // MOD MOD MOD
    U1MODEbits.PDSEL = 0b00;    /* 8-bit data, no parity */
    U1MODEbits.STSEL = 0;       /* 1 stop bit */
    U1MODEbits.UARTEN = 01;     /* enable UART 1 */
    U1STAbits.UTXEN = 1;        /* enable transmitter */
    // U1MODEbits.BRGH = 1;
    U1BRG = BRGVAL;             /* set baudrate */
    // NOTE: automatic baud rate measurement is used due to inaccurate oscillator
    // U1MODEbits.ABAUD = 1;       /* enable automatic baud rate measurement */

    /*------------------------------------------------------------------------*/
    /* Configuration of IO ports */
    /*------------------------------------------------------------------------*/
    /* LED1 */
    TRISBbits.TRISB6 = 0;  /* digital output */
    LATBbits.LATB6 = 0;    /* LED off */
    /* LED2 */
    TRISBbits.TRISB7 = 0;  /* digital output */
    LATBbits.LATB7 = 0;    /* LED off */
    /* LED3 */
    TRISBbits.TRISB8 = 0;  /* digital output */
    LATBbits.LATB8 = 0;    /* LED off */
    /* LED4 */
    TRISBbits.TRISB9 = 0;  /* digital output */
    LATBbits.LATB9 = 0;    /* LED off */
    
    
    //CVRCONbits.CVR2OE = 0;
    ANSELAbits.ANSA4 = 0;
    TRISAbits.TRISA4 = 1;   // PB2 digital input
    TRISBbits.TRISB5 = 1;   // PB1 digital input
    

    /* UART1 Rx */
    ANSELBbits.ANSB2 = 0;
    ANSELBbits.ANSB3 = 0;
    TRISBbits.TRISB2 = 1;  /* RB11: digital input (UART Rx configuration is done in serialInit()) */
    TRISBbits.TRISB3 = 0;
    
  //  CNPU1bits.CN11PUE = 1;  /* activate weak pull up */
  //  CNPUBbits.CNPUB11 = 1; // MOD dsPIC33FJ -> dsPIC33EP
    
    
    /* VPOT */
    // AD1PCFGLbits.PCFG0 = 0; /* AN0: analog input */
    // MOD dsPIC33FJ -> dsPIC33EP
    ANSELAbits.ANSA0 = 1;
    /* VTEMP */
   // AD1PCFGLbits.PCFG1 = 0; /* AN1: analog input */
    // MOD dsPIC33FJ -> dsPIC33EP
    ANSELAbits.ANSA1 = 1;
            
    /* PIEZO */
//    TRISBbits.TRISB5 = 0;       /* digital ouput */
//    // RPOR2bits.RP5R = 0b10010;   /* map OC1 to RP5 */
//    // dsPIC33F -> RP5 = PIN14, dsPIC33E PIN14 = RP37
//    RPOR1bits.RP37R = 0b010000;
    
    /* Encoder */
//    RPINR14bits.QEA1R = 38;     /* map QEA1 to RP38 */
//    RPINR14bits.QEB1R = 39;     /* map QEB1 to RP39 */

    /*------------------------------------------------------------------------*/
    /* Configuration of ADC */
    /*------------------------------------------------------------------------*/
    AD1CON1bits.AD12B = 0;      /* 10bit mode */
    AD1CON2bits.VCFG = 0b000;   /* AVdd and AVss as voltage reference */
    AD1CON3bits.ADRC = 1;       /* ADC uses internal RC clock */
    AD1CON2bits.CHPS = 0b00;    /* converts CH0 */
    AD1CHS0bits.CH0SA = 0b0001; /* channel 0 positive input is AN0 */
    AD1CON1bits.SIMSAM = 1;     /* simultaneously sampling */
    AD1CON1bits.ASAM = 1;       /* sampling begins after last conversion */
    AD1CON1bits.SSRC = 0b111;   /* internal counter ends sampling and starts conversion (auto conversion) */
    AD1CON3bits.SAMC = 0b00000; /* not used for SSRC = 0b111 */
    AD1CON1bits.FORM = 0b00;    /* integer output (right aligned) */
    AD1CON1bits.ADON = 1;       /* enable ADC */

    /*------------------------------------------------------------------------*/
    /* Configuration of QEP unit */
    /*------------------------------------------------------------------------*/
    
 //   QEI1CONbits.QEIM = 0b111;   /* QEI enabled (x4 mode) with counter reset by match */
 //   MAX1CNT = 0x7FFF;           /* only positive numbers [Q15] */

    // MOD MOD MOD
    // QEI Init
    /* Mode 6: limit count range to between QEI1LEC and QEI1GEC */
//    QEI1CON = 0x0;
//    QEI1CONbits.PIMOD = 6;
//
//    /* Limit count range between 0 and PULSES_PER_REV */
//    QEI1LECL = 0;
//    QEI1LECH = 0;
//    QEI1GECL = 0;
//    QEI1GECH = 1;
//
//    QEI1IOC = 0x0070; // noninverted index, QEA, QEB pins
//    QEI1CONbits.QEIEN = 1;
    /*------------------------------------------------------------------------*/
    /* Configuration of timer1 unit for sampling time */
    /*------------------------------------------------------------------------*/
    T1CONbits.TCS = 0;      /* internal clock (fCY) */
    T1CONbits.TGATE = 0;    /* timer in counter mode */
    T1CONbits.TCKPS = 0b00; /* 1:1 prescaler */
    PR1 = 4000;             /* @ fCY = 40MHz -> 100�s timebase */
    TMR1 = 0x0000;          /* clear timer 1 value */
    IFS0bits.T1IF = 0;      /* clear interrupt flag */
    IEC0bits.T1IE = 1;      /* set interrupt enable bit */
    T1CONbits.TON = 1;      /* start timer 1 */


//    /*------------------------------------------------------------------------*/
//    /* Configuration of timer2 unit for compare unit (PWM) */
//    /*------------------------------------------------------------------------*/
//    T2CONbits.TCS = 0;      /* internal clock (fCY) */
//    T2CONbits.TGATE = 0;    /* timer in counter mode */
//    T2CONbits.TCKPS = 0b00; /* 1:1 prescaler */
//    PR2 = PWM_FREQUENCY;    /* @ fCY = 40MHz -> 4kHz PWM frquency */
//    TMR2 = 0x0000;          /* clear timer 2 value */
//    T2CONbits.TON = 1;      /* start timer 2 */

    /*------------------------------------------------------------------------*/
    /* Configuration of compare unit for PWM */
    /*------------------------------------------------------------------------*/
    
  //  OC1CONbits.OCTSEL = 0;      /* timer 2 as clock source */
  //  OC1R = 0x0000;              /* clear compare value */
  //  OC1RS = 0x0000;             /* clear secondary compare value */
  //  OC1CONbits.OCM = 0b110;     /* PWM mode, fault pin disabled */
    
//    // MOD MOD MOD
//    
//    OC1CON1bits.OCTSEL = 0;      /* timer 2 as clock source */
//    OC1R = 0x0000;              /* clear compare value */
// //   OC1RS = 0x0000;             /* clear secondary compare value */ //MOD
//    OC1CON1bits.OCM = 0b110;     /* PWM mode, fault pin disabled */

}


/******************************************************************************/
/* Initialisation of serial interface (overriding methods) */
/******************************************************************************/
void initSerial(tSerial* serial)
{
    serial->send = (void (*)(tInterface*, uint8))sendSerial;
    serial->receive = (uint8 (*)(tInterface*))receiveSerial;
    serial->isReceiveDataAvailable = (uint8 (*)(tInterface*))isReceiveDataAvailable;
    serial->isSendReady = (uint8 (*)(tInterface*))isSendReady;
}

static void sendSerial(tSerial* serial, uint8 data)
{
    U1TXREG = data;
}

static uint8 receiveSerial(tSerial* serial)
{
    if (U1STA & 0x0E)
    {
        U1STAbits.OERR = 0; /* reset error */
        return ((uint8)0);
    }
    return (U1RXREG);
}

static uint8 isReceiveDataAvailable(tSerial* serial)
{
    return (U1STAbits.URXDA);
}

static uint8 isSendReady(tSerial* serial)
{
    return (!U1STAbits.UTXBF);
}

/*
 * Interrupt service routine for timer 1.
 */
void __attribute__((__interrupt__, no_auto_psv)) _T1Interrupt(void)
{
    IFS0bits.T1IF = 0; /* clear interrupt flag */
    mainTask();
}
