/** @file
 * @brief Collection of globally needed defines.
 *
 * Available Preprocessor Definitions:
 * - "__40MHZ__"			Processor clock is 40MHz (standard)
 * - "__70MHZ__"			Processor clock is 70MHz
 * - "__EP100__"			Motor type is EP100
 * - "__EP200__"			Motor type is EP200
 * .
 */
/* 
 * $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#ifndef __GLOBALDEFINES_H__
	#define __GLOBALDEFINES_H__
	
    #include <p33Exxxx.h>
	#include "Target.h"
	#include "X2C.h"

	/*------------------------------------------------------------------------*/
	/* Type Definitions */
	/*------------------------------------------------------------------------*/
    /** Frequency inverter states */
	typedef enum {
        RESET_STATE,            /**< Inverter is in reset/idle state */
        POWER_UP_STATE_1,       /**< Inverter is powering up power module (step 1) */
        POWER_UP_STATE_2,       /**< Inverter is powering up power module (step 2) */
        POWER_UP_STATE_3,       /**< Inverter is powering up power module (step 3) */
        POWER_UP_STATE_4,       /**< Inverter is powering up power module (step 4) */
        POWER_UP_STATE_5,       /**< Inverter is powering up power module (step 5) */
        POWER_UP_STATE_6,       /**< Inverter is powering up power module (step 6) */
        RUN_STATE,              /**< Inverter is operational */
        WAIT_AND_RESET_VOLTAGE, /**< Inverter waits for DC-link voltage to decrease */
        WAIT_4_STALL,           /**< Inverter waits for motor stand still */
        ERROR_STATE             /**< Inverter is in error state */
    } tFI_STATES;

	/*------------------------------------------------------------------------*/
    /* Watchdog */
	/*------------------------------------------------------------------------*/
    #define KICK_DOG \
        do { \
            ClrWdt(); \
        } while (0)


	/*------------------------------------------------------------------------*/
	/** @name X2C Outports
     * @{ */
	/*------------------------------------------------------------------------*/
	#define PWM_PHASE_1		(*Outports.pPhase1)		/* Duty cycle for phase 1 */
	#define PWM_PHASE_2		(*Outports.pPhase2)		/* Duty cycle for phase 2 */
	#define PWM_PHASE_3   	(*Outports.pPhase3)		/* Duty cycle for phase 3 */
	#define PWM_ENABLE		(*Outports.pEnablePWM)  /* Enable signal to power up motor driver */
    /** @} */

	/*------------------------------------------------------------------------*/
	/** @name X2C Inports
     * @{ */
	/*------------------------------------------------------------------------*/
	#define I_PHASE_1		(Inports.I_Phase1)		/* Phase current 1 */
	#define I_PHASE_2		(Inports.I_Phase2)		/* Phase current 2 */
	#define I_PHASE_3		(Inports.I_Phase3)		/* Phase current 3 */
    #define U_HALL_1        (Inports.U_Hall1)       /* Angular hall sensor signal 1 (sine) */
    #define U_HALL_2        (Inports.U_Hall2)       /* Angular hall sensor signal 2 (cosine) */
	#define U_DCLINK		(Inports.U_DCLink)		/* DC-Link voltage */
	#define INT_ENABLE		(Inports.Enable)		/* Enable for controller blocks */

    #define DIGITAL_IN      (Inports.DigitalIn)
    #define ILIMIT_OUT      (Inports.ILimitDriver)
    #define ZC_OUT          (Inports.ZCDriver)

    #define SPEED_REF       (Inports.SpeedRef)       /* External reference speed */
    /** @} */

	/*------------------------------------------------------------------------*/
	/** @name Generic Error Codes (must not be changed!)
     * @{ */
	/*------------------------------------------------------------------------*/
	#define ERROR_CURRENT				0x0001U		/* General current error */
	#define ERROR_VOLTAGE				0x0010U		/* General voltage error */
	#define ERROR_TEMPERATURE			0x0100U		/* General temperature error */
	#define ERROR_COMMUNICATION			0x1000U		/* General communication error */
	#define	ERROR_DSP					0x2000U		/* General DSP error */
	#define ERROR_APPLICATION			0x4000U		/* General application error */
	#define NO_ERROR					0x0000U		/* No error */
    /** @} */

	/*------------------------------------------------------------------------*/
	/** @name Application Specific Error Codes
     * @{ */
	/*------------------------------------------------------------------------*/
	#define ERROR_OVER_CURRENT			0x0001U		/* Over current error (software, fast response) */
	#define ERROR_OVER_CURRENT2			0x0002U		/* Over current error (software, slow response) */
	#define ERROR_CURRENT_MEASUREMENT	0x0004U		/* Current sensor error */
	#define ERROR_TRIPZONE				0x0008U		/* Over current error (hardware) */
	#define ERROR_UDC_MAX				0x0010U		/* DC-Link over voltage error */
	#define ERROR_UDC_MIN				0x0020U		/* DC-Link under voltage error */
	#define ERROR_TMP_HS				0x0100U		/* Heatsink temperature error (not used) */
	#define ERROR_TMP_PCB				0x0200U		/* PCB temperature error */
	#define ERROR_MODULE				0x1000U		/* Power module error */
	#define ERROR_EXTERN				0x2000U		/* External error */
	#define ERROR_POSITION				0x4000U		/* Position sensor error (not used) */
    /** @} */

	/*------------------------------------------------------------------------*/
	/** @name PWM Frequencies
     * @{ */
	/*------------------------------------------------------------------------*/
	/* value = fCY / fPWM */
	#if defined(__70MHZ__)

		#define PWM_10KHZ			7000U	/* PWM frequency = 10 kHz */
		#define PWM_16KHZ			4375U	/* PWM frequency = 16 kHz */
		#define PWM_20KHZ			3500U	/* PWM frequency = 20 kHz */
		#define PWM_30KHZ			2333U	/* PWM frequency = 30 kHz */
		#define PWM_40KHZ			1750U	/* PWM frequency = 40 kHz */

		#if defined(__USE_V0_SVM__)
			#define PWM_10KHZ_MAX_ON	6825U	/* 97.5% of max output */
			#define PWM_16KHZ_MAX_ON	4266U	/* 97.5% of max output */
			#define PWM_20KHZ_MAX_ON	3413U	/* 97.5% of max output */
			#define PWM_30KHZ_MAX_ON	2275U	/* 97.5% of max output */
			#define PWM_40KHZ_MAX_ON	1706U	/* 97.5% of max output */
		#else
			#define PWM_10KHZ_MAX_ON	6650U	/* 95% of max output */
			#define PWM_16KHZ_MAX_ON	4156U	/* 95% of max output */
			#define PWM_20KHZ_MAX_ON	3325U	/* 95% of max output */
			#define PWM_30KHZ_MAX_ON	2217U	/* 95% of max output */
			#define PWM_40KHZ_MAX_ON	1663U	/* 95% of max output */
		#endif

	#elif defined(__40MHZ__)

		#define PWM_10KHZ			4000U	/* PWM frequency = 10 kHz */
		#define PWM_16KHZ			2500U	/* PWM frequency = 16 kHz */
		#define PWM_20KHZ			2000U	/* PWM frequency = 20 kHz */
		#define PWM_30KHZ			1333U	/* PWM frequency = 30 kHz */
		#define PWM_40KHZ			1000U	/* PWM frequency = 40 kHz */

		#if defined(__USE_V0_SVM__)
			#define PWM_10KHZ_MAX_ON	3900U	/* 97.5% of max output */
			#define PWM_16KHZ_MAX_ON	2438U	/* 97.5% of max output */
			#define PWM_20KHZ_MAX_ON	1950U	/* 97.5% of max output */
			#define PWM_30KHZ_MAX_ON	1300U	/* 97.5% of max output */
			#define PWM_40KHZ_MAX_ON	 975U	/* 97.5% of max output */
		#else
			#define PWM_10KHZ_MAX_ON	3800U	/* 95% of max output */
			#define PWM_16KHZ_MAX_ON	2375U	/* 95% of max output */
			#define PWM_20KHZ_MAX_ON	1900U	/* 95% of max output */
			#define PWM_30KHZ_MAX_ON	1267U	/* 95% of max output */
			#define PWM_40KHZ_MAX_ON	 950U	/* 95% of max output */
		#endif

	#else
		#error PROCESSOR CLOCK NOT DEFINED
	#endif

	/* desired PWM frequency */
	// TODO: PLEASE SELECT YOUR DESIRED PWM FREQUENCY!
	#define PWM_FREQUENCY		PWM_10KHZ         /* selected PWM frequency */
	#define PWM_MAX_ON			PWM_10KHZ_MAX_ON
    /** @} */

	/*------------------------------------------------------------------------*/
	/** @name Sample time prescaler
     * @{ */
	/*------------------------------------------------------------------------*/
    #define ONE_PERIOD      (0b0000)        /* trigger ADC every single PWM period -> Ts = 1/PWM_FREQUENCY (NOTE: not allowed, see Hardware.c!) */
    #define TWO_PERIOD      (0b0001)		/* trigger ADC every second PWM period -> Ts = 2/PWM_FREQUENCY */

	/* desired ADC trigger rate */
	// TODO: PLEASE SELECT YOUR DESIRED ADC TRIGGER RATE!
	#define PERIOD_SELECT	TWO_PERIOD      /* selected sample time prescaler */
    /** @} */


	/*------------------------------------------------------------------------*/
	/* Macro for timing values
     * 
	 * input value:  desired time in ms
	 * return value: calculated (counter) value to reach desired time 
	 * (info: Ts = 2 * 1/PWM_FREQUENCY)
     */ 
	/*------------------------------------------------------------------------*/
	#if PWM_FREQUENCY == PWM_10KHZ
		#if PERIOD_SELECT == ONE_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 10U)
		#elif PERIOD_SELECT == TWO_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 5U)
		#else
			#error PERIOD SELECT NOT DEFINED!
		#endif
	#elif PWM_FREQUENCY == PWM_16KHZ
		#if PERIOD_SELECT == ONE_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 16U)
		#elif PERIOD_SELECT == TWO_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 8U)
		#else
			#error PERIOD SELECT NOT DEFINED!
		#endif
	#elif PWM_FREQUENCY == PWM_20KHZ
		#if PERIOD_SELECT == ONE_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 20U)
		#elif PERIOD_SELECT == TWO_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 10U)
		#else
			#error PERIOD SELECT NOT DEFINED!
		#endif
	#elif PWM_FREQUENCY == PWM_30KHZ
		#if PERIOD_SELECT == ONE_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 30U)
		#elif PERIOD_SELECT == TWO_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 15U)
		#else
			#error PERIOD SELECT NOT DEFINED!
		#endif
	#elif PWM_FREQUENCY == PWM_40KHZ
		#if PERIOD_SELECT == ONE_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 40U)
		#elif PERIOD_SELECT == TWO_PERIOD
			#define TIME_MS(val)	((uint16)(val) * 20U)
		#else
			#error PERIOD SELECT NOT DEFINED!
		#endif
	#else
		#error PWM FREQUENCY NOT DEFINED!
	#endif

    
    /*------------------------------------------------------------------------*/
	/** @name Port Pin Definitions
     * @{ */
	/*------------------------------------------------------------------------*/
    #if defined(__EP100__)
        #define READ_ILIMIT_OUT     (PORTAbits.RA9)
    #elif defined(__EP200__)
        #define READ_ILIMIT_OUT     (PORTCbits.RC13)
    #endif
    #define READ_DIGITAL_IN         (PORTCbits.RC8)
    #define READ_ZC_OUT             (PORTAbits.RA8)
    #define GET_LIN_FAULT           (PORTBbits.RB4)
    #define ENABLE_DRIVER           (LATAbits.LATA7 = 1)
    #define DISABLE_DRIVER          (LATAbits.LATA7 = 0)

    #define ENABLE_PWM_PINS do{\
                                IOCON1 |= 0xC000;\
                                IOCON2 |= 0xC000;\
                                IOCON3 |= 0xC000;\
                            }while(0)

    #define DISABLE_PWM_PINS do{\
                                IOCON1 &= 0x3FFF;\
                                IOCON2 &= 0x3FFF;\
                                IOCON3 &= 0x3FFF;\
                            }while(0)

    #define TURN_OFF_LOW_SIDE   do{\
                                    LATBbits.LATB15 = 0;\
                                    LATBbits.LATB13 = 0;\
                                    LATBbits.LATB11 = 0;\
                                }while(0)
    #define TURN_ON_LOW_SIDE    do{\
                                    LATBbits.LATB15 = 1;\
                                    LATBbits.LATB13 = 1;\
                                    LATBbits.LATB11 = 1;\
                                }while(0)
    /** @} */

    
	/*------------------------------------------------------------------------*/
	/* CPU Load Measurement Macros                                            */
	/*------------------------------------------------------------------------*/
	#define START_CPU_MEASUREMENT	(TMR1 = 0x0000U)
	#define END_CPU_MEASUREMENT		(CPU_Load = TMR1)
	#define START_CPU_MEASUREMENT2	(TMR2 = 0x0000U)
	#define END_CPU_MEASUREMENT2	(CPU_Load2 = TMR2)


	/*------------------------------------------------------------------------*/
	/* Global Variables                                                       */
	/*------------------------------------------------------------------------*/
	extern volatile tTableStruct *TableStruct;	/* Pointer to TableStruct */
	extern tFI_STATES FIState;					/* State of frequency inverter */

	extern volatile int16 IrqSyncFlag;			/* Flag used for periodic call of X2C_Update() */
	extern uint16 CalibrationCounter;			/* Enable calibration */
    extern uint16 ADCValue[6];                  /* ADC results */
	extern uint16 ADCOffset[3];					/* Offset values of current measurement */
	extern uint16 CPU_Load;						/* CPU load */
    extern uint16 CPU_Load2;					/* CPU load for slow task */
	extern uint16 CPU_Scale;					/* CPU load scale factor (equals 100%) */
    extern volatile uint8 DriverState[5];       /* Current status of motor driver */
	//extern uint8 PositionError;				/* Flag to signalize problem with position sensor */

#endif
