/* $LastChangedRevision: 819 $
 * $LastChangedDate:: 2015-11-05 15:25:38 +0100 $
 *
 * Copyright (c) 2016, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "GlobalDefines.h"
#include "MCP8025.h"
#include "Main.h"
#include "InterruptControl.h"


/** 
 * @brief Default interrupt catches any non-registered interrupt.
 */
void __attribute__((interrupt, no_auto_psv)) _DefaultInterrupt(void)
{
    while(1); /* wait until watchdog resets CPU */
}


/**
 * @brief Interrupt service routine for ADC end of conversion interrupt.
 * 
 * This routine will be called every 100us (= sample time) and calls the main
 * application task.
 */
void __attribute__((__interrupt__, no_auto_psv)) _AD1Interrupt(void)
{
    static uint8 cnt = 0;
    IFS0bits.AD1IF = 0;     /* clear interrupt flag */
    IrqSyncFlag = 1;        /* set synchronization flag */ 
    
    /** - start CPU load measurement */
    START_CPU_MEASUREMENT;

    /** - save measurement values */
    ADCValue[1] = ((int32)ADC1BUF3 + (int32)ADC1BUF7)>>1; /* phase current 2 */
    ADCValue[2] = ((int32)ADC1BUF1 + (int32)ADC1BUF5)>>1; /* phase current 3 */
    ADCValue[3] = ADC1BUF6; /* hall sensor sine (last of two samples) */
    ADCValue[4] = ADC1BUF0; /* hall sensor cosine */
    ADCValue[5] = ADC1BUF4; /* DC-link voltage */
    
    /** - execute main control task */
    mainTask();
    
    /** - trigger slow control task */
    cnt++;
    if (cnt >= 2)
    {
        TMR4 = 0xFFFF;  /* on next timer step overflow interrupt will be triggered */
        cnt = 0;
    }
    
    /** - end CPU load measurement */
    END_CPU_MEASUREMENT;
}


/** 
 * @brief Interrupt service routine for Timer 4 overflow interrupt.
 * 
 * This routine will be called every 200us (= 2*sample time) and calls the slow
 * application task.
 */
void __attribute__((interrupt, no_auto_psv)) _T4Interrupt(void)
{
    IFS1bits.T4IF = 0;  /* clear interrupt flag */
    
    /** - start CPU load measurement 2 */
    START_CPU_MEASUREMENT2;
    /** -  execute slow control task */
    slowTask();         
    
    /** - end CPU load measurement 2 */
    END_CPU_MEASUREMENT2;
}


/**
 * @brief Interrupt service routine for UART 2 receive interrupt.
 * 
 * This routine will be called when a message from the motor driver is received.
 */
void __attribute__((interrupt, no_auto_psv)) _U2RXInterrupt(void)
{
    uint8 data = 0;
    
    IFS1bits.U2RXIF = 0; /* clear UART 2 Rx interrupt flag bit */

    /* fetch and process data */
    data = U2RXREG;
    readDriverData(data);
}

/**
 * @brief Interrupt service routine for UART 2 transmit interrupt.
 * 
 * This routine will be called when a message to the motor driver is sent.
 */
void __attribute__((interrupt, no_auto_psv)) _U2TXInterrupt(void)
{
    uint8 dummy = 0;
    
    IFS1bits.U2TXIF = 0;    /* clear UART 2 Tx interrupt flag bit */
    IEC1bits.U2TXIE = 0;    /* disable UART 2 Tx interrupt */
    
    /** - disable UART transmitter */
    U2STAbits.UTXEN = 0x00;

    /** - flush UART receive buffer */
    while (U2STAbits.URXDA)
    {
        dummy = U2RXREG;            
    }
 
    /** - enable UART receiver */
    IFS1bits.U2RXIF = 0;            /* clear UART 2 Rx interrupt flag bit */
    IEC1bits.U2RXIE = 1;            /* enable UART 2 Rx interrupt */
}

/**
 * @brief Interrupt service routine for UART 1 error interrupt.
 * 
 * This routine will be called when a UART 1 error occurs. The error will be re-
 * set and the receive buffer emptied.
 */
 void __attribute__ ((interrupt, no_auto_psv)) _U1ErrInterrupt(void)
 {
     uint8 dummy;
     
     IFS4bits.U1EIF = 0;    /* clear UART 1 error interrupt flag */
     U1STAbits.OERR = 0;    /* reset overrun error flag */
     dummy = U1RXREG;       /* read receive register to reset remaining errors */
 }
