/*
 * Handling of 3-phase motor driver.
 * - setup and initialization of driver
 * - sending data
 * - receiving data
 * .
 */
/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "GlobalDefines.h"
#include "MCP8025.h"

/* private macros */
#define STATUS_0    1
#define STATUS_1    2
#define CONFIG_0    3
#define CONFIG_1    4
#define CONFIG_2    5

/* private prototypes */
void writeDriverDataBlocking(uint8 data);
void waitForDriver(void);


/**
 * @brief Routine to setup up motor driver MCP8025.
 */
void setupDriver()
{
    /* Initialize motor driver */
    writeDriverDataBlocking(0x81);      /* command to write configuration register 0 */
    writeDriverDataBlocking(0x00);      /* configuration values 0: default values (0.25V overcurrent limit) */
    writeDriverDataBlocking(0x82);      /* send request for configuration values 0 */
    waitForDriver();
    
    writeDriverDataBlocking(0x83);      /* command to write configuration register 1 */
    writeDriverDataBlocking(212);       /* configuration values 1: 3.910V -> 30A overcurrent limit */
    writeDriverDataBlocking(0x84);      /* send request for configuration values 1 */
    waitForDriver();
    
    writeDriverDataBlocking(0x87);      /* command to write configuration register 2 */
    writeDriverDataBlocking(0b00010000);/* configuration values 2: 1000ns dead time, 4us blanking time */    
    writeDriverDataBlocking(0x88);      /* send request for configuration values 2 */
    waitForDriver();
    
    writeDriverDataBlocking(0x85);      /* send request for status values 0 */
    waitForDriver();
    
    writeDriverDataBlocking(0x86);      /* send request for status values 1 */
    waitForDriver();   
}

/**
 * @brief Routine to process data from motor driver.
 * 
 * @param[in] data Received data byte to be processed.
 */
void readDriverData(uint8 data)
{
    static uint8 FirstByte = 0;
    
    if (FirstByte == 0) /* data byte is first byte in stream (= header byte) */
    {
        if ((data == 0x45) || (data == 0x85))
        {
            /* Status 0 header received */
            FirstByte = STATUS_0;
        }
        else if ((data == 0x46) || (data == 0x86))
        {
            /* Status 1 header received */
            FirstByte = STATUS_1;
        }
        else if ((data == 0x41) || (data == 0x42))
        {
            /* Config 0 header received */
            FirstByte = CONFIG_0;
        }
        else if ((data == 0x43) || (data == 0x44))
        {
            /* Config 1 header received */
            FirstByte = CONFIG_1;
        }
        else if ((data == 0x47) || (data == 0x48))
        {
            /* Config 2 header received */
            FirstByte = CONFIG_2;
        }
        else if (data <= 0x08)
        {
            /* not acknowledged response -> transmission error occured */
            FirstByte = 0;  /* reset header byte */
        }
    }
    else /* data byte is second byte in stream (= data byte) */
    {
        DriverState[FirstByte-1] = data;
        FirstByte = 0;  /* reset header byte */ 
    }
}


/**
 * @brief Routine to send data to motor driver.
 * 
 * @param[in] data Data byte to send.
 */
void writeDriverData(uint8 data)
{
    /** - disable UART receiver */
    while(!U2STAbits.RIDLE);        /* wait until UART receiver is idle */
    IEC1bits.U2RXIE = 0;            /* disable UART 2 Rx interrupt */

    /** - enable UART transmitter */
    U2STAbits.UTXEN = 0x01;
    IEC1bits.U2TXIE = 1;            /* enable UART 2 Tx interrupt */
    
    /** - transmit data */
    while(U2STAbits.UTXBF);         /* wait until UART transmit buffer can be filled */
    U2TXREG = data;                 /* fill shift register with data */

    /** - disabling of transmitter and enabling of receiver is done in UART Tx
          interrupt service routine */
}

/**
 * @brief Routine to send data to motor driver.
 * 
 * During the write the processor will be blocked until transmissen has finished. 
 * 
 * @param[in] data Data byte to send.
 */
void writeDriverDataBlocking(uint8 data)
{
    uint8 dummy;
    
    /** - disable UART receiver */
    while(!U2STAbits.RIDLE);        /* wait until UART receiver is idle */
    IEC1bits.U2RXIE = 0;            /* disable UART 2 Rx interrupt */

    /** - enable UART transmitter */
    U2STAbits.UTXEN = 0x01;         
    
    /** - transmit data */
    while(U2STAbits.UTXBF);         /* wait until UART transmit buffer can be filled */
    U2TXREG = data;                 /* fill shift register with data */    
    while(!U2STAbits.TRMT);         /* wait until shift register is empty -> data has been sent */
     
    /** - disable UART transmitter */
    U2STAbits.UTXEN = 0x00;
  
    /** - flush UART receive buffer */
    while (U2STAbits.URXDA)
    {
        dummy = U2RXREG;            
    }
 
    /** - enable UART receiver */
    IFS1bits.U2RXIF = 0;            /* clear UART 2 Rx interrupt flag bit */
    IEC1bits.U2RXIE = 1;            /* enable UART 2 Rx interrupt */
    
}

/**
 * @brief Time delay function.
 * 
 * This function delays the program by 3ms. This time should be sufficient for
 * the MCP8025 motor driver to send a 2 byte answer.
 */
void waitForDriver(void)
{
    uint16 cnt = 0;
    while (cnt < TIME_MS(5))        /* 2 * 10 bits = 2.1ms @ 9600bps */
	{
		while (!IrqSyncFlag) {};	/* wait until ADC conversion has finished */
    	cnt++;
        
		KICK_DOG;
		IrqSyncFlag = 0;
	}
}