/*
 * Initialization and main function.
 */
/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "TableStruct.h"
#include "LNet.h"
#include "Services.h"
#include "BlockServicesPIC33E.h"
#include "SerialPIC33E.h"
#include "CheckValidApplication.h"
#include "VersionInfo.h"
#include "GlobalDefines.h"
#include "Hardware.h"
#include "MCP8025.h"
#include "SystemControl.h"
#include "InputControl.h"
#include "OutputControl.h"
#include "Main.h"

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint8)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];


/* private prototypes */
void doMonitorLoop(void);

/*  static variables */
volatile int8 InitializationFinished = 0;

/** 
 * @brief Main function.
 * 
 * @return The main function will never return due to the never ending loop.
 */
int main(void)
{
    /** - Initialize "integrated monitor":
     *    - Configuration of LNet protocol:
     *      - Node-ID: 1
     *      - Buffer size: 255
     *      .
     *    - Configuration of serial interface:
     *      - Baudrate: 115.2kB/s
     *      - Data bits: 8
     *      - Parity: none
     *      - Stop bits: 1
     *      .
     *    .
     */
    initTableStruct();
    initSerialA(&interface, BAUDRATE_115200, DATABITS_8, STOPBITS_1, PARITY_OFF, RXPIN_RPI53, TXPIN_RP20);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);
    TableStruct->DSPState = PRG_LOADED_STATE;
    if (checkValidApplication()) {
        TableStruct->DSPState = PRG_LOADED_STATE;
    } else {
        doMonitorLoop();
    }

    initVersionInfo(TableStruct, (uint16)07);   /* Software version 0.7 */
    TableStruct->TFncTable = blockFunctionTable;
    TableStruct->TParamTable = parameterIdTable;
    KICK_DOG;
    
    /** - Initialize hardware */
    initHardware();
    initReverseProtectionChargePump();
    KICK_DOG;
    
    /** - Initialize motor driver */
    setupDriver();
    KICK_DOG;
      
    /** - Initialize X2C */
    X2C_Init();
    KICK_DOG;

    TableStruct->DSPState = IDLE_STATE;
    TableStruct->eventId   = NO_ERROR;
    TableStruct->eventType = NO_ERROR;

	/** - Initialize CPU load measurement */
	#if PERIOD_SELECT == ONE_PERIOD
		CPU_Scale = PWM_FREQUENCY;
	#elif (PERIOD_SELECT == TWO_PERIOD)
		CPU_Scale = PWM_FREQUENCY << 1;
	#else
		#error PERIOD_SELECT NOT DEFINED/FOUND!
	#endif

    /** - Calibrate current measurement */
	while (CalibrationCounter != (uint16)INT16_MAX)		/* wait until calibration has finished */
	{
		while (!IrqSyncFlag) {};	/* wait until ADC conversion has finished */
    	calibrateADC();

		/* Communication */
		protocol.pCommunicate((tProtocol*)&protocol);
        
		KICK_DOG;
		IrqSyncFlag = 0;
	}
    InitializationFinished = 1;		/* signalize that initialization is over */
    
    /** - Never ending loop -> interrupt driven algorithm */
    while (1)
    {
        static uint16 cnt = 0;
        
        /* Communication */
        protocol.pCommunicate((tProtocol*)&protocol);
        KICK_DOG;
        
        if (IrqSyncFlag)
        {
            cnt++;
//            /* Communication with motor driver */
//            if (cnt == TIME_MS(100))
//            {
//                writeDriverData(0x85);      /* send request for status values 0 */
//            }
//            if (cnt >= TIME_MS(200))
//            {
//                writeDriverData(0x86);      /* send request for status values 1 */
//                cnt = 0;
//            }
            KICK_DOG;
            IrqSyncFlag = 0;
            
        }
    }
    return (0);
}

/** 
 * @brief Main control task.
 * 
 *  Calling rate = 100us
 */
void mainTask(void)
{  
    if (InitializationFinished != 0)
	{
        /** - read inports */
		readADC();				/* read analog inputs */
		readDigitalIn();		/* read digital inputs */
        
        /** - State machine update and error check */
		updateSystemControl();	/* safety functions, module startup */

        /** - update X2C */
	    switch (TableStruct->DSPState)
	    {
	        case IDLE_STATE :
	            break;
	        case INIT_STATE :
	            X2C_Init();
	            TableStruct->DSPState = IDLE_STATE;
				TableStruct->eventId   = NO_ERROR;
				TableStruct->eventType = NO_ERROR;
	            break;
	        case RUN_STATE_POWER_OFF :
	        	X2C_Update_1();
	            break;
	        case RUN_STATE_POWER_ON :
			    X2C_Update_1();
				break;
	        default:
	            TableStruct->DSPState = PRG_LOADED_STATE;
	            break;
		}

        /** - write outports */
		setPWM();				/* set PWM values */
		KICK_DOG;
	}
}


/**
 * @brief Slow control task.
 * 
 * Calling rate: 2 * 100us 
 */
void slowTask(void){

    /** - call X2C update functions */
    //X2C_Update_2();         
    KICK_DOG;
}


/** Monitor loop for communication */
void doMonitorLoop(void)
{
    while (1)
    {
        KICK_DOG;
        protocol.pCommunicate((tProtocol*)&protocol);
    }
}
