/* $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25 +0100#$
 *
 * Copyright (c) 2015 Linz Center of Mechatronics GmbH
 * All rights reserved.
 */
#include "SerialPIC33E.h"

/* private prototypes */
static void setupSerial(tSerial* serialPIC33E, tSerialBaudrate baudrate);
static void sendSerial(tSerial* serialPIC33E, uint8 data);
static uint8 receiveSerial(tSerial* serialPIC33E);
static uint8 isReceiveDataAvailable(tSerial* serialPIC33E);
static uint8 isSendReady(tSerial* serialPIC33E);
static uint8 getTxFifoFree(tSerial* serialPIC33E);
static void flush(tSerial* serialPIC33E);

/**
 * @brief Initialization of UART 1
 * 
 * @param[out] serial33E Serial interface structure
 * @param[in] baudrate Rate of data transfer
 * @param[in] databits Number of data bits
 * @param[in] stopbits Number of stop bits
 * @param[in] parity Type of parity mode
 * @param[in] rxPin Pin for serial receive
 * @param[in] txPin Pin for serial transmit
 */
void initSerialA(tSerial* serial33E, tSerialBaudrate baudrate, tSerialDataBits databits, \
        tSerialStopBits stopbits, tSerialParity parity, tSerialRxPin rxPin, tSerialTxPin txPin)
{
    switch (rxPin)
    {
#if !defined(__DSPIC33EP256GP502__)
        case RXPIN_RPI53:
            /* map UART #1 rx to RPI53 */
            RPINR18bits.U1RXR = (uint8)53;
            break;
#endif
        case RXPIN_RP41:
            /* map UART #1 rx to RP41 */
            RPINR18bits.U1RXR = (uint8)41;
            break;
        default:
            /* don't initialize RX pin */
            break;
    }

    switch (txPin)
    {
        case TXPIN_RP20:
            RPOR0bits.RP20R = (uint8)1;
            break;
        case TXPIN_RP40:
            RPOR3bits.RP40R = (uint8)1;
            break;
        default:
            /* don't initialize TX pin */
            break;
    }

    switch (stopbits)
    {
        default:
        case STOPBITS_1:
            U1MODEbits.STSEL = (uint8)0;
            break;
        case STOPBITS_2:
            U1MODEbits.STSEL = (uint8)1;
            break;
    }

    /* data bits and parity settings depend on each other */
    /* in this driver the data bits setting overrules parity setting */
    switch (databits)
    {
        default:
        case DATABITS_8:
            /* allowed parity modes for 8 data bits */
            switch (parity)
            {
                default:
                case PARITY_OFF:
                    U1MODEbits.PDSEL = (uint8)0x00;
                    break;
                case PARITY_ODD:
                    U1MODEbits.PDSEL = (uint8)0x02;
                    break;
                case PARITY_EVEN:
                    U1MODEbits.PDSEL = (uint8)0x01;
                    break;
            }
            break;
        case DATABITS_9:
            /* only no parity mode allowed for 9 data bits */
            U1MODEbits.PDSEL = (uint8)0x03;
            break;
    }

    serial33E->module = &UART1;
    setupSerial(serial33E, baudrate);
}


/**
 * @brief Linking of serial interface to hardware interface.
 * 
 * @param[out] lnkProtocol Serial protocol structure
 * @param[in] serial33E Serial interface structure
 */
void linkSerial(tProtocol* lnkProtocol, tSerial* serial33E)
{
    lnkProtocol->hwInterface = (tInterface*)serial33E;
}

static void sendSerial(tSerial* serialPIC33E, uint8 data)
{
    U1TXREG = data;
}

static uint8 receiveSerial(tSerial* serialPIC33E)
{
    /* read register to clear pending errors */
    U1STA;
    return (U1RXREG);
}

static uint8 isReceiveDataAvailable(tSerial* serialPIC33E)
{
    return (U1STAbits.URXDA);
}

static uint8 isSendReady(tSerial* serialPIC33E)
{
    return (!U1STAbits.UTXBF);
}

static uint8 getTxFifoFree(tSerial* serialPIC33E)
{
    return ((uint8)0);
}

static void flush(tSerial* serialPIC33E)
{
    /* do nothing because 'send' immediately sends byte */
}

static void setupSerial(tSerial* serialPIC33E, tSerialBaudrate baudrate)
{
    switch (baudrate) {
#if defined(__70MHZ__)
        case BAUDRATE_19200:
            U1BRG = (uint16)227;
            break;
        default:
        case BAUDRATE_38400:
            U1BRG = (uint16)113;
            break;
        case BAUDRATE_115200:
            U1BRG = (uint16)37;
            break;
#elif defined(__40MHZ__)
        case BAUDRATE_19200:
            U1BRG = (uint16)129;
            break;
        default:
        case BAUDRATE_38400:
            U1BRG = (uint16)64;
            break;
        case BAUDRATE_115200:
            U1BRG = (uint16)21;
            break;
#else
#error CLOCK NOT DEFINED
#endif
    }

    /* enable UART #1 module and UART transmitter */
    U1MODEbits.UARTEN = 0x01;
    U1STAbits.UTXEN = 0x01;

    serialPIC33E->send = (void (*)(tInterface*, uint8))sendSerial;
    serialPIC33E->receive = (uint8 (*)(tInterface*))receiveSerial;
    serialPIC33E->isReceiveDataAvailable = (uint8 (*)(tInterface*))isReceiveDataAvailable;
    serialPIC33E->isSendReady = (uint8 (*)(tInterface*))isSendReady;
    serialPIC33E->getTxFifoFree = (uint8 (*)(tInterface*))getTxFifoFree;
    serialPIC33E->flush = (void (*)(tInterface*))flush;
}
