/* 
 * $LastChangedRevision: 10 $
 * $LastChangedDate:: 2016-01-13 08:47:25 +0100#$
 *
 * Copyright (c) 2013, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
#include "GlobalDefines.h"
#include "Hardware.h"
#include "SystemControl.h"


/*----------------------------------------------------------------------------*/
/** @name Temperature Limits
 *---------------------------------------------------------------------------- 
 * Scaling PCB temperature EP200: 1 = 500°C
 * @{ */
/*----------------------------------------------------------------------------*/
#if defined(__EP200__)
    #define	TMP_PCB_MAX	5243 		/* 80°C:	-> 5243 = 80/500*32768 */
#endif
/** @} */


/*----------------------------------------------------------------------------*/
/** @name Voltage Limits
 *---------------------------------------------------------------------------- 
 * Scaling DC-Link voltage EP100: 1 = 33.3V
 * @{ */
/* TODO: PLEASE ADAPT THE VOLTAGE LIMITS TO YOUR NEEDS! */
/*----------------------------------------------------------------------------*/
#define UDC_MAX_OFF     14760  			/* 15V:  -> 15/33.3*32768 */
 #define UDC_MAX_ON     13777  			/* 14V:  -> 14/33.3*32768 */
 #define UDC_MIN_OFF	9840  			/*  9V: ->   9/33.3*32768 */
 #define UDC_MIN_ON		8856  			/* 10V: ->  10/33.3*32768 */

#define WAIT_TIME		(TIME_MS(1000))	/* 1s wait time until reset */
/** @} */


/*----------------------------------------------------------------------------*/
/** @name Current Limits
 *---------------------------------------------------------------------------- 
 * Scaling EP200: 1 = 30.05A
 * @{ */
/* TODO: PLEASE ADAPT THE CURRENT LIMITS TO YOUR NEEDS! */
/*----------------------------------------------------------------------------*/
#define CURRENT_MAX			21845			/* 20A:  -> 20/30.0*32768 */
#define CURRENT_MAX_TIME	(TIME_MS(2))	/* 2ms */

#define CURRENT_MAX2		19661			/* 18A:  -> 18/30.0*32768 */
#define CURRENT_MAX2_TIME	(TIME_MS(100))	/* 0.1s */

/* Current measurement offset limit (1 = 5.0V) */
#define CURRENT_OFFSET_MIN	28836U		/* 2.2V: -> 2.2/5.0*65536 */
#define CURRENT_OFFSET_MAX	36700U		/* 2.8V: -> 2.8/5.0*65536 */
/** @} */


/**
 * @brief Update of state machine.
 */
void updateSystemControl(void)
{
	static uint16 Cnt = 0U;
	static int16  Cnt_OC = 0;
	static int16  Cnt_OC2 = 0;
	int16 Enabled;

	/* check if the user wants to startup the power module */
	Enabled = (TableStruct->DSPState == RUN_STATE_POWER_ON) && (PWM_ENABLE != 0);

    switch (FIState)
    {
    	/*--------------------------------------------------------------------*/
        case RESET_STATE :  /** - Reset/Idle state <ul> */

            /** <li> disable driver */
			DISABLE_DRIVER;
            
            /** <li> reset enable signal */
			INT_ENABLE = 0;

			if (Enabled)
			{
				/** <li> check DC-link voltage */
				if (U_DCLINK > UDC_MAX_ON)
				{
					TableStruct->eventId   = ERROR_UDC_MAX;
					TableStruct->eventType = ERROR_VOLTAGE;
					FIState = ERROR_STATE;
				}
				else if (U_DCLINK < UDC_MIN_ON)
				{
					TableStruct->eventId   = ERROR_UDC_MIN;
					TableStruct->eventType = ERROR_VOLTAGE;
					FIState = ERROR_STATE;
				}
				else
				{
					FIState = POWER_UP_STATE_1;
					TableStruct->eventId   = NO_ERROR;
					TableStruct->eventType = NO_ERROR;
                    DriverState[0] = 0;
                    DriverState[1] = 0;
					Cnt_OC = 0;
					Cnt_OC2 = 0;
                    Cnt = 0;

					/** <li> check current measurement offset */                  
                    /* offset phase 2 */
                    if ((ADCOffset[1] < CURRENT_OFFSET_MIN) || (ADCOffset[1] > CURRENT_OFFSET_MAX))
                    {
                        TableStruct->eventId   |= ERROR_CURRENT_MEASUREMENT;
                        TableStruct->eventType |= ERROR_CURRENT;
                        FIState = ERROR_STATE;
                    }
                    /* offset phase 3 */
                    if ((ADCOffset[2] < CURRENT_OFFSET_MIN) || (ADCOffset[2] > CURRENT_OFFSET_MAX))
                    {
                        TableStruct->eventId   |= ERROR_CURRENT_MEASUREMENT;
                        TableStruct->eventType |= ERROR_CURRENT;
                        FIState = ERROR_STATE;
                    }
				}
			}
			break; /** </ul> */
        /*--------------------------------------------------------------------*/
		case POWER_UP_STATE_1 : /** - Power up state - enable driver */
			
            TURN_OFF_LOW_SIDE;  /* switch low side MOSFETs off */
            DISABLE_PWM_PINS;   /* switch to GPIO pins */
            ENABLE_DRIVER;      /* enable power module */
            
			Cnt++;
			if (Cnt >= TIME_MS(15))		/* 15 ms (MCP2805 datasheet: typ 10ms to power up) */
			{
				FIState = POWER_UP_STATE_2;
				Cnt = 0U;
			}	
			break;
        /*--------------------------------------------------------------------*/
		case POWER_UP_STATE_2 : /** - Power up state - reset driver fault */
			
            DISABLE_DRIVER;     /* disable power module */
            
			Cnt++;
			if (Cnt > 1)		/* 1 * Ts */
			{
                ENABLE_DRIVER;  /* enable power module */
				FIState = POWER_UP_STATE_3;
				Cnt = 0U;
			}
			break;
		/*--------------------------------------------------------------------*/
        case POWER_UP_STATE_3 : /** - Power up state - load bootstrap caps */
            
            TURN_ON_LOW_SIDE;  /* switch low side MOSFETs on -> start loading */
       
			Cnt++;
			if (Cnt >= TIME_MS(3))		/* 3ms to load capacitors */
			{
				FIState = POWER_UP_STATE_4;
				Cnt = 0U;
			}	
			break;
		/*--------------------------------------------------------------------*/
		case POWER_UP_STATE_4 : /** - Power up state - enable PWM units */
			
            ENABLE_PWM_PINS;    /* switch to PWM pins */           
            TURN_OFF_LOW_SIDE;  /* switch low side MOSFETs off (not effective) */
            
            INT_ENABLE = 0;		/* reset X2C enable signal */
			Cnt++;
			if (Cnt >= 1)		/* 1 * Ts */
			{
				FIState = RUN_STATE;
				Cnt = 0U;
			}	
			break;
		/*--------------------------------------------------------------------*/
		case  RUN_STATE:        /** - Operational state <ul> */

			if (Enabled)
			{
				/** <li> set enable signal for certain X2C blocks */
				INT_ENABLE = INT8_MAX;

    			/** <li> check DC-Link voltage for over voltage */
				if (U_DCLINK > UDC_MAX_OFF)
				{
					TableStruct->eventId   |= ERROR_UDC_MAX;
					TableStruct->eventType |= ERROR_VOLTAGE;
				}

				/** <li> check DC-Link voltage for under voltage */
				if (U_DCLINK < UDC_MIN_OFF)
				{
					TableStruct->eventId   |= ERROR_UDC_MIN;
					TableStruct->eventType |= ERROR_VOLTAGE;
				}

				/** <li> check for power module failure */
				if (!READ_ILIMIT_OUT)
				{
                    TableStruct->eventId   |= ERROR_MODULE;
                    TableStruct->eventType |= ERROR_DSP;
				}

				/** <li> check phase currents (fast response) */
                if ((I_PHASE_1 > CURRENT_MAX) || (I_PHASE_1 < -CURRENT_MAX) || \
					(I_PHASE_2 > CURRENT_MAX) || (I_PHASE_2 < -CURRENT_MAX) || \
                    (I_PHASE_3 > CURRENT_MAX) || (I_PHASE_3 < -CURRENT_MAX))
				{
					Cnt_OC++;
					if (Cnt_OC > (int16)CURRENT_MAX_TIME)
					{
						TableStruct->eventId   |= ERROR_OVER_CURRENT;
						TableStruct->eventType |= ERROR_CURRENT;
					}
				}
				else if (Cnt_OC > 0)
				{
					Cnt_OC--;
				}

				/** <li> check phase currents (slow response) */
                if ((I_PHASE_1 > CURRENT_MAX2) || (I_PHASE_1 < -CURRENT_MAX2) || \
					(I_PHASE_2 > CURRENT_MAX2) || (I_PHASE_2 < -CURRENT_MAX2) || \
                    (I_PHASE_3 > CURRENT_MAX2) || (I_PHASE_3 < -CURRENT_MAX2))
				{
					Cnt_OC2++;
					if (Cnt_OC2 > (int16)CURRENT_MAX2_TIME)
					{
						TableStruct->eventId   |= ERROR_OVER_CURRENT2;
						TableStruct->eventType |= ERROR_CURRENT;
					}
				}
				else if (Cnt_OC2 > 0)
				{
					Cnt_OC2--;
				}

				/** <li> check for errors/warnings */
				if (TableStruct->eventId & 0xFFFFFFFFU)		/* ERROR */
				{
					FIState = ERROR_STATE;
					DISABLE_DRIVER;
					Cnt = 0U;
				}
			}
			else
			{
				FIState = RESET_STATE;
				DISABLE_DRIVER;
				Cnt = 0U;
			}
			break; /** </ul> */
		/*--------------------------------------------------------------------*/
		case WAIT_AND_RESET_VOLTAGE:	/** - Wait for voltage to reset (not used)  */
			
			Cnt++;
			if (Cnt >= WAIT_TIME)
			{
				FIState = RESET_STATE;
			}
			break;
		/*--------------------------------------------------------------------*/
		case ERROR_STATE :              /** - Wait for error to be reset */

			DISABLE_DRIVER;

    		if (!Enabled)
			{
				FIState = RESET_STATE;
			}
			break;
		/*--------------------------------------------------------------------*/
		default:
			DISABLE_DRIVER;
			break;
		/*--------------------------------------------------------------------*/
	}	

}
