/* FOC Demo Application, tested with:
 *      MCLV2
 *      Hurst Brushless
 *      dsPIC33EP256MC506
 */

#include "TableStruct.h"
#include "LNet.h"
#include "Services.h"
#include "BlockServicesX2C.h"
#include "Hardware.h"
#include "Main.h"

#include <xc.h>

#define SET_LED_D2        (LATDbits.LATD6 = 1)
#define CLEAR_LED_D2     (LATDbits.LATD6 = 0)
#define TOGGLE_LED_D2   (LATDbits.LATD6 ^= 1)

#define SET_LED_D17        (LATDbits.LATD5 = 1)
#define CLEAR_LED_D17     (LATDbits.LATD5 = 0)
#define TOGGLE_LED_D17   (LATDbits.LATD5 ^= 1)


/* used by init version info */
const struct {
	uint8 date[11];
	uint8 time[8];
} compilationDate = { __DATE__, __TIME__ };

/* X2C maintenance structure */
volatile tTableStruct TheTableStruct;
volatile tTableStruct *TableStruct = &TheTableStruct;

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint8)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];

int cnt_div = 20;

int16 pwm_max =  0.9f * LOOPINTCY;
int16 pwm_min =  0.1f * LOOPINTCY;

/* private prototypes */
static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion);

/*
 * Main function.
 */
int main(void)
{
    initTableStruct();
    initSerialGeneric(&interface);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);
    TableStruct->DSPState = PRG_LOADED_STATE;

    initVersionInfo(TableStruct, (uint16)0x0001);
    TableStruct->TFncTable = blockFunctionTable;
    TableStruct->TParamTable = parameterIdTable;

    initSerial(&interface);
    
    X2C_Init();
    KICK_DOG;
       
    initHardware();
    KICK_DOG;

    while (1)
    {
        /* communication */
        protocol.pCommunicate((tProtocol*)&protocol);
    }
    return (0);
}

/*
 * Main control task.
 */
void mainTask(void)
{
    TOGGLE_LED_D2; // debugging calculation-time
    /*------------------------------------------------------------------------*/
    /* assign inports */
    /*------------------------------------------------------------------------*/
    
    // Omega-Setpoint from POT1 on MCLV2
    Inports.omega_set = (int16)ADC1BUF0 >> 2;
    
    // Get Velocity every cnt_div'th cycle
    if(cnt_div == 0) {
        cnt_div = 20;
        Inports.omega_meas = (int16)VEL1CNT * 100;
    }
    cnt_div--;
    
    
    //Motorcurrent: Amplifier on MCLV is inverting
    Inports.i_a = -ADC1BUF1;
    Inports.i_b = -ADC1BUF2;
    
    /* encoder */
    // calculating a linear function to use whole Q15 representation
    int32 qei_calc = ((int32)POS1CNTL-(PULSE_PER_REV/(2*N_P)))*20971;
    Inports.qei = (int16)(qei_calc >> 6); // Calculating QEI in Q9.6

    /*------------------------------------------------------------------------*/
    /* update X2C */
    /*------------------------------------------------------------------------*/
    TOGGLE_LED_D17;
    X2C_Update();
    TOGGLE_LED_D17;

    /*------------------------------------------------------------------------*/
    /* update outports */
    /*------------------------------------------------------------------------*/
    
    /**
     * The following Codesection calculates the duty-cycle values for the 
     * registers PDC1-3, by using a linear function:
     * 
     * f(x) = [(PWM_MAX - PWM-MIN)/(2 *INT16_MAX)] * (x - INT16_MAX) + PWM_MAX
     *                       op_a                           op_b
     * 
     * op_a is a Q15 representation of the fraction (PWM_MAX - PWM-MIN)/(2 *INT16_MAX)
     * and is precalculated for some PWM-Frequencies.
     */

    int16 op_a = 1400; 
    
#ifdef PWM10KHZ
    op_a = 2800; 
#endif
#ifdef PWM15KHZ
    op_a = 1867; 
#endif
    
    // Calculating in Q32.15
    int32 op_b = (int32)*Outports.pPWM1 - INT16_MAX; 
    int64 temp = op_a * op_b;
    PDC1 = (int16)((temp >> 15) + pwm_max);
   
    
    op_b = (int32)*Outports.pPWM2 - INT16_MAX;
    temp = op_a * op_b;
    PDC2 = (int16)((temp >> 15) + pwm_max);
    

    op_b = (int32)*Outports.pPWM3 - INT16_MAX;
    temp = op_a * op_b;
    PDC3 = (int16)((temp>> 15) + pwm_max);

    /* kick watchdog */
    KICK_DOG;
      
    TOGGLE_LED_D2; // debugging calculation-time
}

static void initVersionInfo(volatile tTableStruct* tblStruct, uint16 appVersion)
{
	tblStruct->framePrgVersion = appVersion;
	tblStruct->framePrgCompDateTime = (uint8*)&compilationDate;
   
}
