/* FOC Demo Application, tested with:
 *      MCLV2
 *      Hurst Brushless
 *      dsPIC33EP256MC506
 */

#include "Target.h"
#include "Main.h"
#include "Hardware.h"

/* private prototypes */
static void sendSerial(tSerial* serial, uint8 data);
static uint8 receiveSerial(tSerial* serial);
static uint8 isReceiveDataAvailable(tSerial* serial);
static uint8 isSendReady(tSerial* serial);

/* private macros */
#define F_FRC           8000000         /* fast RC internal oscillator frequency after tuning */
#define F_IN            F_FRC           /* FRC as input for PLL */
#define F_OSC           F_IN*70/(2*2)   /* fOSC = fIN*M/(N1*N2): (PLL) oscillator frequency */
#define F_CY            F_OSC/2         /* instruction cycle frequency */

#define BAUDRATE        115200          /* baudrate for serial interface */
#define BRGVAL          ((F_CY/BAUDRATE)/16)-1

/* ICD Communication Channel Select bits (Communicate on PGEC1 and PGED1) */
#pragma config ICS = PGD2 

/* Select internal FRC with PLL at POR */
_FOSCSEL(FNOSC_FRCPLL & PWMLOCK_OFF); //MOD PWMLOCK
/* Enable Clock Switching, disable primary oscillator and disable clock out function */
_FOSC(FCKSM_CSECMD & POSCMD_NONE & OSCIOFNC_OFF); //MOD

/*
 * Hardware initialization.
 */
void initHardware(void) {
     /* Clock Config */
    /* Configure PLL prescaler, PLL postscaler, PLL divisor
     * FRC frequency: 7.37MHz -> tuning to 8MHz
     * NOTE: FRC frequency is not accurate -> frequency may vary!
     * fCY = 8MHz * 70/(2*2) / 2 = 70Mhz
     */
   
    // 7.37MHz + 23*0.00375*7.37MHz = 8MHz
    OSCTUNbits.TUN = 23; 

     // M = 70
    PLLFBD = 70;

    // N2 = 2 
    CLKDIVbits.PLLPOST = 0; 
    // N1 = 2
    CLKDIVbits.PLLPRE = 0; 

    // Switch to FRC with PLL 
    __builtin_write_OSCCONH(0x01);
    __builtin_write_OSCCONL(0x01);
    
    // Wait for Clock switch to occur 
    while (OSCCONbits.COSC != 0b001);

    // Wait for PLL to lock 
    while (OSCCONbits.LOCK != 1) {
    };
    /*END: Clock Config*/

    
    /* Analog Inputs */
    // AN0: analog input, Motorcurrent
    ANSELAbits.ANSA0 = 1;
    // AN1: analog input, Motorcurrent
    ANSELAbits.ANSA1 = 1;
    // AN13: POTI on MCLV2
    ANSELEbits.ANSE13 = 1;
    /*END: Analog Inputs*/

    
    
    
    /* ADC CONFIG */
    AD1CHS0bits.CH0SA = 13; // AN13 --> Channel 0    
    AD1CON1bits.FORM = 3; // ADC-result format,  11 = Signed fractional (DOUT = sddd dddd dd00 0000, where s = .NOT.d<9>)
    // PWM starts and ends conversion
    AD1CON1bits.SSRCG = 0;
    AD1CON1bits.SSRC = 3;

    // Simultaneous Sampling enabled (only applicable when CHPS<1:0> = 01 or 1x) 
    AD1CON1bits.SIMSAM = 1;

    // Sampling begins immediately after last conversion completes.
    AD1CON1bits.ASAM = 1;

    // Samples CH0, CH1, CH2, CH3 simultaneously (when CHPS = 1x)
    AD1CON2bits.CHPS = 2;

    // A/D Conversion Clock Select bits = 6 * Tcy
    AD1CON3bits.ADCS = 6;
    
    // Clear the A/D interrupt flag bit 
    IFS0bits.AD1IF = 0;
    // Enable A/D interrupt
    IEC0bits.AD1IE = 1;  

    // Turn on A/D module
    AD1CON1bits.ADON = 1;
    /* END: ADC Config*/
    
    
    
    
    /* Digital Pins Config*/
    // Debug LED D2
    TRISDbits.TRISD6 = 0; 
    // Debug LED D17
    TRISDbits.TRISD5 = 0; 
    /* END: Digital Pins Config*/


    
    
    /* UART Config*/
    // RX input - SCL1/RPI53/RC5
    TRISCbits.TRISC5 = 1; 
    
    // TX output - RP97/RF1
    TRISFbits.TRISF1 = 0; 
    
    // Make Pin RP53 U1RX
    RPINR18bits.U1RXR = 53; 
    
    // Make Pin RP97 U1TX
    RPOR7bits.RP97R = 1; 
    
    //set baudrate
    U1BRG = BRGVAL;
    
    // 8-bit data, no parity
    U1MODEbits.PDSEL = 0b00; 
    
    // 1 stop bit
    U1MODEbits.STSEL = 0; 
    
    //enable UART1
    U1MODEbits.UARTEN = 01;
    
    // enable transmitter
    U1STAbits.UTXEN = 1;
    /* END: UART Config */

    
    
    
     /* PWM Config*/
    // Center aligned PWM.
    
    // PWM frequency
    PHASE1 = LOOPINTCY;
    PHASE2 = LOOPINTCY;
    PHASE3 = LOOPINTCY;
    PTPER = 2 * LOOPINTCY + 1; 

    // Enable PWM output pins and configure them as complementary mode
    PWMCON1 = 0x0204; 
    PWMCON2 = 0x0204; 
    PWMCON3 = 0x0204;

    //I/O pins controlled by PWM
    IOCON1 = 0xC000;
    IOCON2 = 0xC000;
    IOCON3 = 0xC000;

    // Deadtime setting
    ALTDTR1 = DDEADTIME; 
    ALTDTR2 = DDEADTIME; 
    ALTDTR3 = DDEADTIME; 

    //fault disabled
    FCLCON1 = 0x0003;
    FCLCON2 = 0x0003;
    FCLCON3 = 0x0003;
    
    // Divide clock  by 1 to generate PWM
    PTCON2 = 0x0000; 

    // low side turn on errata workaround, PDC cannot be initialized with 0 
    PDC1 = PWM_MIN; 
    PDC2 = PWM_MIN;
    PDC3 = PWM_MIN;

    // PWM Interrupt Priority 4
    IPC23bits.PWM1IP = 4; 
    IPC23bits.PWM2IP = 4; 
    IPC24bits.PWM3IP = 4; 

    //clear PWM interrupt flag
    IFS5bits.PWM1IF = 0;
    // Disable PWM interrupts
    IEC5bits.PWM1IE = 0; 

    // Enable PWM for center aligned operation
    PTCON = 0x8000; 

    // SEVTCMP: Special Event Compare Count Register
    // Phase of ADC capture set relative to PWM cycle: 0 offset and counting up
    SEVTCMP = 0;
    /* END: PWM Config*/
    
    
    
    
     /* QEI Config*/
    
    // Data Direction for QEI
    TRISCbits.TRISC6 = 1;
    TRISFbits.TRISF0 = 1;
    
    // PPS for QEI
    RPINR14bits.QEA1R = 54;
    RPINR14bits.QEB1R = 96;
    
    QEI1CON = 0x0;
    QEI1CONbits.PIMOD = 6;

    // Limit count range between 0 and PULSES_PER_REV 
    QEI1LECL = 0;
    QEI1LECH = 0;
    QEI1GECL = 200 - 1;
    QEI1GECH = 0;
    
    // Inverted index, QEA, QEB pins
    QEI1IOC = 0x70; 
    QEI1IOCbits.SWPAB = 1;
   
    // Enable digital filtering, this is a must!
    QEI1IOCbits.FLTREN = 1; 
    
    // QEI enable
    QEI1CONbits.QEIEN = 1;
    /* END: QEI Config*/

    
    
    
    /* Enable watchdog */
    RCONbits.SWDTEN = 1;
}


/******************************************************************************/
/* Initialisation of serial interface (overriding methods) */

/******************************************************************************/
void initSerial(tSerial* serial) {
    serial->send = (void (*)(tInterface*, uint8))sendSerial;
    serial->receive = (uint8(*)(tInterface*))receiveSerial;
    serial->isReceiveDataAvailable = (uint8(*)(tInterface*))isReceiveDataAvailable;
    serial->isSendReady = (uint8(*)(tInterface*))isSendReady;
}

static void sendSerial(tSerial* serial, uint8 data) {
    U1TXREG = data;
}

static uint8 receiveSerial(tSerial* serial) {
    if (U1STA & 0x0E) {
        U1STAbits.OERR = 0; /* reset error */
        return ((uint8) 0);
    }
    return (U1RXREG);
}

static uint8 isReceiveDataAvailable(tSerial* serial) {
    return (U1STAbits.URXDA);
}

static uint8 isSendReady(tSerial* serial) {
    return (!U1STAbits.UTXBF);
}

/*
 * Interrupt service routine for timer 1.
 */
void __attribute__((interrupt, no_auto_psv)) _AD1Interrupt(void) {
    IFS0bits.AD1IF = 0; /* clear interrupt flag */
    mainTask();
}

